﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Diagnostics;
using System.Xml;
using System.Globalization;
using System.Web;

namespace BenjaminSchroeter.GeoNames
{
    public static partial class GeoNamesOrgWebservice
    {
        #region FindCountries
        /// <summary>
        /// List countries of the world for specific langague using web service from geonames.org
        /// </summary>
        /// <param name="langague"></param>
        /// <returns>generic list with properties: Name, Code, isoNumber, and isoAlphs</returns>
        public static List<Countries> FindCountries(string langague )
        {
            if(string.IsNullOrEmpty(langague))langague = "en";
            string xml = FindCountriesXML(langague);

            return CreateCountriesList(xml);
        }

        /// <summary>
        /// List countries of the world for specific langague using web service from geonames.org
        /// </summary>
        /// <param name="langague"></param>
        /// <returns>XML document. see web site for list of elements</returns>
        private static string FindCountriesXML(string langague )
        {
            if(string.IsNullOrEmpty(langague))langague = "en";
            NumberFormatInfo format = new NumberFormatInfo { NumberDecimalSeparator = "." };
            string url = String.Format(format, "http://ws.geonames.org/countryInfo?lang={0}", langague);

            Debug.WriteLine("Requesting " + url);

            string xml = CreateWebClient().DownloadString(url);
            GeonameException.ThrowOnError(xml);
            
            return xml;
        }
                
        /// <summary>
        /// Creates a generic list from XML
        /// </summary>
        /// <param name="xml"></param>
        /// <returns>List</returns>
        private static List<Countries> CreateCountriesList(string xml)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(xml);
            if (doc.DocumentElement == null)
                return new List<Countries>();

            XmlNodeList root = doc.GetElementsByTagName("geonames");
            if (root.Count == 0)
                return new List<Countries>();

            XmlElement geonames = (XmlElement)root[0];
            List<Countries> results = new List<Countries>();
            foreach (XmlElement element in geonames.GetElementsByTagName("country"))
            {
                Countries n = new Countries(element);
                results.Add(n);
            }

            return results;
        }
        #endregion

    }

    /// <summary>
    /// 
    /// </summary>
    
    [DebuggerDisplay("{Title} ({Language})")]
    public class Countries : GeonameDataBase
    {
        /* here are the xml elements passed to list
            <countryCode>AD</countryCode> 
            <countryName>Andorre</countryName> 
            <isoNumeric>20</isoNumeric> 
            <isoAlpha3>AND</isoAlpha3> 
        */
        /// <summary>
        /// 
        /// </summary>
        #region properties
        [Category("Names")]
        public string Name { get; private set; }
        [Category("Names")]
        public string Code { get; private set; }
        [Category("Names")]
        public string isoNumber { get; private set; }
        public string isoAlpha { get; private set; }

        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="genonameXml"></param>
        public Countries(string genonameXml)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(genonameXml);

            if (doc.DocumentElement == null)
                return;

            XmlNodeList root = doc.GetElementsByTagName("entry");
            if (root.Count == 0)
                return;

            xml = (XmlElement)root[0];

            setProperties();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="geoname"></param>
        public Countries(XmlElement geoname)
        {
            xml = geoname;
            setProperties();
        }

        /// <summary>
        /// 
        /// </summary>
        private void setProperties()
        {
            /*
                <countryCode>AD</countryCode> 
                <countryName>Andorre</countryName> 
                <isoNumeric>20</isoNumeric> 
                <isoAlpha3>AND</isoAlpha3> 
            */

            this.Name = GetElement(xml, "countryName");
            this.Code = GetElement(xml, "countryCode");
            this.isoNumber = GetElement(xml, "isoNumeric");
            this.isoAlpha = GetElement(xml, "isoAlpha3");
        }
    }
}
